@@ -12,8 +12,11 @@ use crate::server::protocol::{
     DELTA_MIN_SIZE,
 };
 use crate::ssh::config::SshConfig;
+use crate::sync::live_progress::ProgressState;
 use crate::sync::scanner::{self, ScanOptions};
-use crate::sync::SyncStats;
+use crate::sync::{
+    ChangeAction, DirectoryChange, DryRunDetails, FileChange, SymlinkChange, SyncStats,
+};
 use crate::transport::server::ServerSession;
 
 /// Minimum size for compression (1MB)
@@ -35,12 +38,17 @@ struct SourceEntry {
 }
 
 /// Sync from local source to remote destination using server protocol
-pub async fn sync_server_mode(source: &Path, dest: &SyncPath) -> Result<SyncStats> {
+pub async fn sync_server_mode(
+    source: &Path,
+    dest: &SyncPath,
+    dry_run: bool,
+    progress: Option<Arc<ProgressState>>,
+) -> Result<SyncStats> {
     let start = Instant::now();
 
     // Connect to server
     let mut session = connect(dest).await?;
-    tracing::debug!("Connected to server");
+    tracing::debug!("Connected to server (dry_run: {})", dry_run);
 
     // Scan source
     tracing::debug!("Scanning source...");
@@ -85,23 +93,40 @@ pub async fn sync_server_mode(source: &Path, dest: &SyncPath) -> Result<SyncStat
         total_symlinks
     );
 
+    // Initialize detailed dry-run tracking
+    let mut file_changes: Vec<FileChange> = Vec::new();
+    let mut dir_changes: Vec<DirectoryChange> = Vec::new();
+    let mut symlink_changes: Vec<SymlinkChange> = Vec::new();
+
     // Step 1: Create directories (if any)
     let mut dirs_created = 0u64;
     if !directories.is_empty() {
-        tracing::debug!("Creating {} directories...", directories.len());
-        session.send_mkdir_batch(directories).await?;
-        let ack = session.read_mkdir_ack().await?;
-        dirs_created = ack.created as u64;
-        if !ack.failed.is_empty() {
-            for (path, err) in &ack.failed {
-                tracing::warn!("Failed to create dir {}: {}", path, err);
+        if dry_run {
+            tracing::debug!("[DRY-RUN] Would create {} directories", directories.len());
+            dirs_created = directories.len() as u64;
+            // Track directory changes
+            for dir in &directories {
+                dir_changes.push(DirectoryChange {
+                    path: PathBuf::from(dir),
+                    action: ChangeAction::Create,
+                });
+            }
+        } else {
+            tracing::debug!("Creating {} directories...", directories.len());
+            session.send_mkdir_batch(directories).await?;
+            let ack = session.read_mkdir_ack().await?;
+            dirs_created = ack.created as u64;
+            if !ack.failed.is_empty() {
+                for (path, err) in &ack.failed {
+                    tracing::warn!("Failed to create dir {}: {}", path, err);
+                }
             }
         }
     }
 
     // Step 2: Send file list and get decisions
     tracing::debug!("Sending file list ({} files)...", total_files);
-    session.send_file_list(proto_entries).await?;
+    session.send_file_list(proto_entries.clone()).await?;
 
     tracing::debug!("Waiting for server decisions...");
     let ack = session.read_ack().await?;
@@ -114,10 +139,30 @@ pub async fn sync_server_mode(source: &Path, dest: &SyncPath) -> Result<SyncStat
         .count();
     tracing::info!("{} files need transfer", files_to_transfer);
 
+    // Track skipped files for detailed dry-run
+    if dry_run {
+        for decision in &ack.decisions {
+            if decision.action == Action::Skip {
+                let entry = &files[decision.index as usize];
+                file_changes.push(FileChange {
+                    path: PathBuf::from(&entry.rel_path),
+                    action: ChangeAction::Skip,
+                    size: entry.size,
+                    transfer_bytes: 0,
+                    would_use_delta: false,
+                    would_compress: false,
+                    skip_reason: Some("Already up-to-date".to_string()),
+                });
+            }
+        }
+    }
+
     // Step 3: Transfer files - separate creates and updates
     let mut bytes_transferred = 0u64;
     let mut files_created = 0u64;
     let mut files_updated = 0u64;
+    let mut bytes_would_add = 0u64;
+    let mut bytes_would_change = 0u64;
 
     // Categorize by action type
     let creates: Vec<(u32, &SourceEntry)> = ack
@@ -144,124 +189,50 @@ pub async fn sync_server_mode(source: &Path, dest: &SyncPath) -> Result<SyncStat
         })
         .collect();
 
-    // Step 3a: Handle CREATES with full file transfer (+ compression)
-    if !creates.is_empty() {
-        tracing::debug!("Transferring {} new files...", creates.len());
-
-        let paths: Vec<(u32, Arc<PathBuf>, String, u64)> = creates
-            .iter()
-            .map(|(idx, e)| (*idx, e.abs_path.clone(), e.rel_path.clone(), e.size))
-            .collect();
-
-        // Read and optionally compress files
-        let files_data: Vec<(u32, Vec<u8>, u8)> = tokio::task::spawn_blocking(move || {
-            paths
-                .into_iter()
-                .filter_map(|(idx, path, rel_path, size)| {
-                    std::fs::read(&*path).ok().map(|data| {
-                        // Compress if file is large enough and not already compressed
-                        let (send_data, flags) =
-                            if size >= COMPRESS_MIN_SIZE && !is_compressed_extension(&rel_path) {
-                                match compress(&data, Compression::Zstd) {
-                                    Ok(compressed) if compressed.len() < data.len() => {
-                                        (compressed, DATA_FLAG_COMPRESSED)
-                                    }
-                                    _ => (data, 0),
-                                }
-                            } else {
-                                (data, 0)
-                            };
-                        (idx, send_data, flags)
-                    })
-                })
-                .collect()
-        })
-        .await?;
-
-        // Send all creates
-        for (idx, data, flags) in &files_data {
-            bytes_transferred += data.len() as u64;
-            session
-                .send_file_data_with_flags(*idx, 0, *flags, data.clone())
-                .await?;
-        }
-        session.flush().await?;
-
-        // Read confirmations
-        for _ in &files_data {
-            let done = session.read_file_done().await?;
-            if done.status != 0 {
-                tracing::error!("Create failed: index {} status {}", done.index, done.status);
-            } else {
-                files_created += 1;
-            }
-        }
+    // Initialize progress tracking if provided
+    if let Some(ref progress) = progress {
+        let total_bytes: u64 = creates.iter().map(|(_, e)| e.size).sum::<u64>()
+            + updates.iter().map(|(_, e)| e.size).sum::<u64>();
+        let total_transfers = creates.len() + updates.len();
+        progress.set_totals(total_bytes, total_transfers);
     }
 
-    // Step 3b: Handle UPDATES - use delta sync for large files
-    if !updates.is_empty() {
-        let (delta_candidates, full_updates): (Vec<_>, Vec<_>) =
-            updates.iter().partition(|(_, e)| e.size >= DELTA_MIN_SIZE);
+    // Step 3a: Handle CREATES with full file transfer (+ compression)
+    if !creates.is_empty() {
+        if dry_run {
+            tracing::debug!("[DRY-RUN] Would transfer {} new files", creates.len());
+            for (_, entry) in &creates {
+                files_created += 1;
+                bytes_would_add += entry.size;
 
-        // Process delta candidates with pipelined checksum requests
-        if !delta_candidates.is_empty() {
-            tracing::debug!(
-                "Delta syncing {} large files (>{}KB) with pipeline depth {}...",
-                delta_candidates.len(),
-                DELTA_MIN_SIZE / 1024,
-                PIPELINE_DEPTH
-            );
-
-            // Collect pending requests for batching
-            let mut pending: Vec<(u32, &SourceEntry, u32)> = Vec::with_capacity(PIPELINE_DEPTH);
-
-            for (idx, entry) in &delta_candidates {
-                let file_idx = *idx;
-                let block_size = delta_block_size(entry.size);
-
-                // Send checksum request without waiting (no flush)
-                session
-                    .send_checksum_req_no_flush(file_idx, block_size)
-                    .await?;
-                pending.push((file_idx, *entry, block_size));
-
-                // Process batch when full
-                if pending.len() >= PIPELINE_DEPTH {
-                    session.flush().await?;
-                    let (updated, transferred) =
-                        process_delta_batch(&mut session, &pending).await?;
-                    files_updated += updated;
-                    bytes_transferred += transferred;
-                    pending.clear();
-                }
+                // Track detailed file change
+                let would_compress =
+                    entry.size >= COMPRESS_MIN_SIZE && !is_compressed_extension(&entry.rel_path);
+                file_changes.push(FileChange {
+                    path: PathBuf::from(&entry.rel_path),
+                    action: ChangeAction::Create,
+                    size: entry.size,
+                    transfer_bytes: entry.size, // Full transfer for creates
+                    would_use_delta: false,
+                    would_compress,
+                    skip_reason: None,
+                });
             }
+        } else {
+            tracing::debug!("Transferring {} new files...", creates.len());
 
-            // Process remaining files
-            if !pending.is_empty() {
-                session.flush().await?;
-                let (updated, transferred) = process_delta_batch(&mut session, &pending).await?;
-                files_updated += updated;
-                bytes_transferred += transferred;
-            }
-        }
-
-        // Process small file updates with full transfer
-        if !full_updates.is_empty() {
-            tracing::debug!(
-                "Full transfer for {} small file updates...",
-                full_updates.len()
-            );
-
-            let paths: Vec<(u32, Arc<PathBuf>, String, u64)> = full_updates
+            let paths: Vec<(u32, Arc<PathBuf>, String, u64)> = creates
                 .iter()
                 .map(|(idx, e)| (*idx, e.abs_path.clone(), e.rel_path.clone(), e.size))
                 .collect();
 
+            // Read and optionally compress files
             let files_data: Vec<(u32, Vec<u8>, u8)> = tokio::task::spawn_blocking(move || {
                 paths
                     .into_iter()
                     .filter_map(|(idx, path, rel_path, size)| {
                         std::fs::read(&*path).ok().map(|data| {
+                            // Compress if file is large enough and not already compressed
                             let (send_data, flags) = if size >= COMPRESS_MIN_SIZE
                                 && !is_compressed_extension(&rel_path)
                             {
@@ -281,7 +252,19 @@ pub async fn sync_server_mode(source: &Path, dest: &SyncPath) -> Result<SyncStat
             })
             .await?;
 
+            // Send all creates - track progress for each file
+            // We need to map idx back to original entry for progress tracking
+            let idx_to_entry: HashMap<u32, &SourceEntry> =
+                creates.iter().map(|(idx, e)| (*idx, *e)).collect();
+
             for (idx, data, flags) in &files_data {
+                // Start transfer progress
+                if let Some(ref progress) = progress {
+                    if let Some(entry) = idx_to_entry.get(idx) {
+                        progress.start_transfer(PathBuf::from(&entry.rel_path));
+                    }
+                }
+
                 bytes_transferred += data.len() as u64;
                 session
                     .send_file_data_with_flags(*idx, 0, *flags, data.clone())
@@ -289,12 +272,172 @@ pub async fn sync_server_mode(source: &Path, dest: &SyncPath) -> Result<SyncStat
             }
             session.flush().await?;
 
-            for _ in &files_data {
+            // Read confirmations
+            for (idx, _, _) in &files_data {
                 let done = session.read_file_done().await?;
                 if done.status != 0 {
-                    tracing::error!("Update failed: index {} status {}", done.index, done.status);
+                    tracing::error!("Create failed: index {} status {}", done.index, done.status);
                 } else {
-                    files_updated += 1;
+                    files_created += 1;
+                    // Finish transfer progress
+                    if let Some(ref progress) = progress {
+                        if let Some(entry) = idx_to_entry.get(idx) {
+                            progress.finish_transfer(&PathBuf::from(&entry.rel_path), entry.size);
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    // Step 3b: Handle UPDATES - use delta sync for large files
+    if !updates.is_empty() {
+        if dry_run {
+            tracing::debug!("[DRY-RUN] Would update {} files", updates.len());
+            for (_, entry) in &updates {
+                files_updated += 1;
+                bytes_would_change += entry.size;
+
+                // Track detailed file change
+                let would_use_delta = entry.size >= DELTA_MIN_SIZE;
+                let would_compress =
+                    entry.size >= COMPRESS_MIN_SIZE && !is_compressed_extension(&entry.rel_path);
+                // For dry-run, estimate transfer bytes (conservative: assume 50% for delta)
+                let transfer_bytes = if would_use_delta {
+                    entry.size / 2
+                } else {
+                    entry.size
+                };
+                file_changes.push(FileChange {
+                    path: PathBuf::from(&entry.rel_path),
+                    action: ChangeAction::Update,
+                    size: entry.size,
+                    transfer_bytes,
+                    would_use_delta,
+                    would_compress,
+                    skip_reason: None,
+                });
+            }
+        } else {
+            let (delta_candidates, full_updates): (Vec<_>, Vec<_>) =
+                updates.iter().partition(|(_, e)| e.size >= DELTA_MIN_SIZE);
+
+            // Process delta candidates with pipelined checksum requests
+            if !delta_candidates.is_empty() {
+                tracing::debug!(
+                    "Delta syncing {} large files (>{}KB) with pipeline depth {}...",
+                    delta_candidates.len(),
+                    DELTA_MIN_SIZE / 1024,
+                    PIPELINE_DEPTH
+                );
+
+                // Collect pending requests for batching
+                let mut pending: Vec<(u32, &SourceEntry, u32)> = Vec::with_capacity(PIPELINE_DEPTH);
+
+                for (idx, entry) in &delta_candidates {
+                    let file_idx = *idx;
+                    let block_size = delta_block_size(entry.size);
+
+                    // Send checksum request without waiting (no flush)
+                    session
+                        .send_checksum_req_no_flush(file_idx, block_size)
+                        .await?;
+                    pending.push((file_idx, *entry, block_size));
+
+                    // Process batch when full
+                    if pending.len() >= PIPELINE_DEPTH {
+                        session.flush().await?;
+                        let (updated, transferred) =
+                            process_delta_batch(&mut session, &pending, progress.as_ref()).await?;
+                        files_updated += updated;
+                        bytes_transferred += transferred;
+                        pending.clear();
+                    }
+                }
+
+                // Process remaining files
+                if !pending.is_empty() {
+                    session.flush().await?;
+                    let (updated, transferred) =
+                        process_delta_batch(&mut session, &pending, progress.as_ref()).await?;
+                    files_updated += updated;
+                    bytes_transferred += transferred;
+                }
+            }
+
+            // Process small file updates with full transfer
+            if !full_updates.is_empty() {
+                tracing::debug!(
+                    "Full transfer for {} small file updates...",
+                    full_updates.len()
+                );
+
+                let paths: Vec<(u32, Arc<PathBuf>, String, u64)> = full_updates
+                    .iter()
+                    .map(|(idx, e)| (*idx, e.abs_path.clone(), e.rel_path.clone(), e.size))
+                    .collect();
+
+                let files_data: Vec<(u32, Vec<u8>, u8)> = tokio::task::spawn_blocking(move || {
+                    paths
+                        .into_iter()
+                        .filter_map(|(idx, path, rel_path, size)| {
+                            std::fs::read(&*path).ok().map(|data| {
+                                let (send_data, flags) = if size >= COMPRESS_MIN_SIZE
+                                    && !is_compressed_extension(&rel_path)
+                                {
+                                    match compress(&data, Compression::Zstd) {
+                                        Ok(compressed) if compressed.len() < data.len() => {
+                                            (compressed, DATA_FLAG_COMPRESSED)
+                                        }
+                                        _ => (data, 0),
+                                    }
+                                } else {
+                                    (data, 0)
+                                };
+                                (idx, send_data, flags)
+                            })
+                        })
+                        .collect()
+                })
+                .await?;
+
+                // Map idx to entry for progress tracking
+                let idx_to_entry: HashMap<u32, &SourceEntry> =
+                    full_updates.iter().map(|(idx, e)| (*idx, *e)).collect();
+
+                for (idx, data, flags) in &files_data {
+                    // Start transfer progress
+                    if let Some(ref progress) = progress {
+                        if let Some(entry) = idx_to_entry.get(idx) {
+                            progress.start_transfer(PathBuf::from(&entry.rel_path));
+                        }
+                    }
+
+                    bytes_transferred += data.len() as u64;
+                    session
+                        .send_file_data_with_flags(*idx, 0, *flags, data.clone())
+                        .await?;
+                }
+                session.flush().await?;
+
+                for (idx, _, _) in &files_data {
+                    let done = session.read_file_done().await?;
+                    if done.status != 0 {
+                        tracing::error!(
+                            "Update failed: index {} status {}",
+                            done.index,
+                            done.status
+                        );
+                    } else {
+                        files_updated += 1;
+                        // Finish transfer progress
+                        if let Some(ref progress) = progress {
+                            if let Some(entry) = idx_to_entry.get(idx) {
+                                progress
+                                    .finish_transfer(&PathBuf::from(&entry.rel_path), entry.size);
+                            }
+                        }
+                    }
                 }
             }
         }
@@ -303,25 +446,40 @@ pub async fn sync_server_mode(source: &Path, dest: &SyncPath) -> Result<SyncStat
     // Step 4: Create symlinks (if any)
     let mut symlinks_created = 0u64;
     if !symlinks.is_empty() {
-        tracing::debug!("Creating {} symlinks...", symlinks.len());
+        if dry_run {
+            tracing::debug!("[DRY-RUN] Would create {} symlinks", symlinks.len());
+            symlinks_created = symlinks.len() as u64;
+            // Track symlink changes
+            for symlink in &symlinks {
+                if let Some(target) = &symlink.symlink_target {
+                    symlink_changes.push(SymlinkChange {
+                        path: PathBuf::from(&symlink.rel_path),
+                        action: ChangeAction::Create,
+                        target: target.clone(),
+                    });
+                }
+            }
+        } else {
+            tracing::debug!("Creating {} symlinks...", symlinks.len());
 
-        let symlink_entries: Vec<SymlinkEntry> = symlinks
-            .iter()
-            .filter_map(|e| {
-                e.symlink_target.as_ref().map(|target| SymlinkEntry {
-                    path: e.rel_path.clone(),
-                    target: target.clone(),
+            let symlink_entries: Vec<SymlinkEntry> = symlinks
+                .iter()
+                .filter_map(|e| {
+                    e.symlink_target.as_ref().map(|target| SymlinkEntry {
+                        path: e.rel_path.clone(),
+                        target: target.clone(),
+                    })
                 })
-            })
-            .collect();
+                .collect();
 
-        if !symlink_entries.is_empty() {
-            session.send_symlink_batch(symlink_entries).await?;
-            let ack = session.read_symlink_ack().await?;
-            symlinks_created = ack.created as u64;
-            if !ack.failed.is_empty() {
-                for (path, err) in &ack.failed {
-                    tracing::warn!("Failed to create symlink {}: {}", path, err);
+            if !symlink_entries.is_empty() {
+                session.send_symlink_batch(symlink_entries).await?;
+                let ack = session.read_symlink_ack().await?;
+                symlinks_created = ack.created as u64;
+                if !ack.failed.is_empty() {
+                    for (path, err) in &ack.failed {
+                        tracing::warn!("Failed to create symlink {}: {}", path, err);
+                    }
                 }
             }
         }
@@ -338,6 +496,36 @@ pub async fn sync_server_mode(source: &Path, dest: &SyncPath) -> Result<SyncStat
         duration
     );
 
+    // Mark progress as finished
+    if let Some(ref progress) = progress {
+        progress.mark_finished();
+    }
+
+    // Build detailed dry-run info if in dry-run mode
+    let dry_run_details = if dry_run {
+        // Estimate network bytes (actual data transferred over network)
+        let estimated_network_bytes: u64 = file_changes.iter().map(|fc| fc.transfer_bytes).sum();
+
+        // Estimate duration based on average transfer speed (conservative: 10 MB/s)
+        let avg_speed_bytes_per_sec = 10 * 1024 * 1024;
+        let estimated_duration_secs = if estimated_network_bytes > 0 {
+            (estimated_network_bytes as f64 / avg_speed_bytes_per_sec as f64).max(0.1)
+        } else {
+            0.0
+        };
+
+        Some(DryRunDetails {
+            file_changes,
+            directory_changes: dir_changes,
+            symlink_changes,
+            filtered_files: Vec::new(), // TODO: Track filtered files
+            estimated_duration_secs,
+            estimated_network_bytes,
+        })
+    } else {
+        None
+    };
+
     Ok(SyncStats {
         files_scanned: total_files as u64,
         files_created,
@@ -349,6 +537,10 @@ pub async fn sync_server_mode(source: &Path, dest: &SyncPath) -> Result<SyncStat
         errors: Vec::new(),
         dirs_created,
         symlinks_created,
+        bytes_would_add,
+        bytes_would_change,
+        bytes_would_delete: 0,
+        dry_run_details,
         ..Default::default()
     })
 }
@@ -420,12 +612,17 @@ async fn scan_source(source: &Path) -> Result<Vec<SourceEntry>> {
 }
 
 /// Sync from remote source to local destination using server protocol (PULL mode)
-pub async fn sync_pull_server_mode(source: &SyncPath, dest: &Path) -> Result<SyncStats> {
+pub async fn sync_pull_server_mode(
+    source: &SyncPath,
+    dest: &Path,
+    dry_run: bool,
+    progress: Option<Arc<ProgressState>>,
+) -> Result<SyncStats> {
     let start = Instant::now();
 
     // Connect to server in PULL mode
     let mut session = connect_pull(source).await?;
-    tracing::debug!("Connected to server (PULL mode)");
+    tracing::debug!("Connected to server (PULL mode, dry_run: {})", dry_run);
 
     // Ensure local destination exists
     if !dest.exists() {
@@ -444,6 +641,13 @@ pub async fn sync_pull_server_mode(source: &SyncPath, dest: &Path) -> Result<Syn
     let mut files_skipped = 0usize;
     let mut bytes_transferred = 0u64;
     let mut symlinks_created = 0u64;
+    let mut bytes_would_add = 0u64;
+    let mut bytes_would_change = 0u64;
+
+    // Detailed dry-run tracking
+    let mut file_changes: Vec<FileChange> = Vec::new();
+    let mut dir_changes: Vec<DirectoryChange> = Vec::new();
+    let symlink_changes: Vec<SymlinkChange> = Vec::new();
 
     // Step 1: Receive and create directories
     let mkdir_batch = session.read_mkdir_batch().await?;
@@ -451,16 +655,35 @@ pub async fn sync_pull_server_mode(source: &SyncPath, dest: &Path) -> Result<Syn
     let mut dirs_created = 0u64;
     let mut failed: Vec<(String, String)> = Vec::new();
 
-    for dir_path in &mkdir_batch.paths {
-        let full_path = dest.join(dir_path);
-        match std::fs::create_dir_all(&full_path) {
-            Ok(_) => dirs_created += 1,
-            Err(e) => failed.push((dir_path.clone(), e.to_string())),
+    if dry_run {
+        tracing::debug!(
+            "[DRY-RUN] Would create {} directories",
+            mkdir_batch.paths.len()
+        );
+        dirs_created = mkdir_batch.paths.len() as u64;
+        // Track directory changes
+        for dir in &mkdir_batch.paths {
+            dir_changes.push(DirectoryChange {
+                path: PathBuf::from(dir),
+                action: ChangeAction::Create,
+            });
         }
+        // Still need to send ack for protocol
+        session
+            .send_mkdir_batch_ack(dirs_created as u32, failed.clone())
+            .await?;
+    } else {
+        for dir_path in &mkdir_batch.paths {
+            let full_path = dest.join(dir_path);
+            match std::fs::create_dir_all(&full_path) {
+                Ok(_) => dirs_created += 1,
+                Err(e) => failed.push((dir_path.clone(), e.to_string())),
+            }
+        }
+        session
+            .send_mkdir_batch_ack(dirs_created as u32, failed)
+            .await?;
     }
-    session
-        .send_mkdir_batch_ack(dirs_created as u32, failed)
-        .await?;
 
     // Step 2: Receive file list and send decisions
     let file_list = session.read_file_list().await?;
@@ -469,73 +692,163 @@ pub async fn sync_pull_server_mode(source: &SyncPath, dest: &Path) -> Result<Syn
     let mut decisions: Vec<Decision> = Vec::with_capacity(file_list.entries.len());
     let mut files_to_receive: Vec<(u32, String)> = Vec::new();
 
-    for (idx, entry) in file_list.entries.iter().enumerate() {
-        let action = if let Some((local_size, local_mtime)) = local_map.get(&entry.path) {
-            // File exists locally - compare
-            if *local_size == entry.size && *local_mtime >= entry.mtime {
-                Action::Skip
-            } else {
-                Action::Update
-            }
-        } else {
-            Action::Create
-        };
-
-        if action != Action::Skip {
-            files_to_receive.push((idx as u32, entry.path.clone()));
-        } else {
-            files_skipped += 1;
-        }
-
-        decisions.push(Decision {
-            index: idx as u32,
-            action,
-        });
-    }
-
-    tracing::info!(
-        "{} files to receive, {} skipped",
-        files_to_receive.len(),
-        files_skipped
-    );
-    session.send_file_list_ack(decisions).await?;
-
-    // Step 3: Receive files
-    for (idx, rel_path) in &files_to_receive {
-        let file_data = match session.read_file_data().await? {
-            Some(data) => data,
-            None => break, // Server sent symlinks instead
-        };
-
-        let full_path = dest.join(rel_path);
-
-        // Ensure parent directory exists
-        if let Some(parent) = full_path.parent() {
-            std::fs::create_dir_all(parent)?;
-        }
-
-        // Write file
-        match std::fs::write(&full_path, &file_data.data) {
-            Ok(_) => {
-                bytes_transferred += file_data.data.len() as u64;
-                if local_map.contains_key(rel_path) {
-                    files_updated += 1;
+    // Track what WOULD happen in dry-run using metadata (before making decisions)
+    if dry_run {
+        for entry in file_list.entries.iter() {
+            let action = if let Some((local_size, local_mtime)) = local_map.get(&entry.path) {
+                if *local_size == entry.size && *local_mtime >= entry.mtime {
+                    ChangeAction::Skip
                 } else {
-                    files_created += 1;
+                    ChangeAction::Update
                 }
-                session.send_file_done(*idx, 0).await?;
+            } else {
+                ChangeAction::Create
+            };
+
+            // Track using metadata only - no downloads!
+            match action {
+                ChangeAction::Create => {
+                    files_created += 1;
+                    bytes_would_add += entry.size;
+                    file_changes.push(FileChange {
+                        path: PathBuf::from(&entry.path),
+                        action: ChangeAction::Create,
+                        size: entry.size,
+                        transfer_bytes: entry.size,
+                        would_use_delta: false,
+                        would_compress: false,
+                        skip_reason: None,
+                    });
+                }
+                ChangeAction::Update => {
+                    files_updated += 1;
+                    bytes_would_change += entry.size;
+                    file_changes.push(FileChange {
+                        path: PathBuf::from(&entry.path),
+                        action: ChangeAction::Update,
+                        size: entry.size,
+                        transfer_bytes: entry.size,
+                        would_use_delta: false,
+                        would_compress: false,
+                        skip_reason: None,
+                    });
+                }
+                ChangeAction::Skip => {
+                    files_skipped += 1;
+                    file_changes.push(FileChange {
+                        path: PathBuf::from(&entry.path),
+                        action: ChangeAction::Skip,
+                        size: entry.size,
+                        transfer_bytes: 0,
+                        would_use_delta: false,
+                        would_compress: false,
+                        skip_reason: Some("Already up-to-date".to_string()),
+                    });
+                }
+                _ => {}
             }
-            Err(e) => {
-                tracing::warn!("Failed to write {}: {}", full_path.display(), e);
-                session.send_file_done(*idx, 2).await?; // STATUS_WRITE_ERROR
+        }
+
+        // Tell server to skip everything (dry-run)
+        let skip_decisions: Vec<Decision> = (0..file_list.entries.len())
+            .map(|idx| Decision {
+                index: idx as u32,
+                action: Action::Skip,
+            })
+            .collect();
+        session.send_file_list_ack(skip_decisions).await?;
+    } else {
+        // Normal mode: make real decisions
+        for (idx, entry) in file_list.entries.iter().enumerate() {
+            let action = if let Some((local_size, local_mtime)) = local_map.get(&entry.path) {
+                // File exists locally - compare
+                if *local_size == entry.size && *local_mtime >= entry.mtime {
+                    Action::Skip
+                } else {
+                    Action::Update
+                }
+            } else {
+                Action::Create
+            };
+
+            if action != Action::Skip {
+                files_to_receive.push((idx as u32, entry.path.clone()));
+            } else {
+                files_skipped += 1;
+            }
+
+            decisions.push(Decision {
+                index: idx as u32,
+                action,
+            });
+        }
+
+        tracing::info!(
+            "{} files to receive, {} skipped",
+            files_to_receive.len(),
+            files_skipped
+        );
+
+        // Initialize progress tracking if provided
+        if let Some(ref progress) = progress {
+            // Calculate total bytes from files to receive
+            let total_bytes: u64 = files_to_receive
+                .iter()
+                .filter_map(|(idx, _)| file_list.entries.get(*idx as usize))
+                .map(|e| e.size)
+                .sum();
+            progress.set_totals(total_bytes, files_to_receive.len());
+        }
+
+        session.send_file_list_ack(decisions).await?;
+
+        // Step 3: Receive files (only in non-dry-run mode)
+        for (idx, rel_path) in &files_to_receive {
+            // Start transfer progress
+            if let Some(ref progress) = progress {
+                progress.start_transfer(PathBuf::from(rel_path));
+            }
+            let file_data = match session.read_file_data().await? {
+                Some(data) => data,
+                None => break, // Server sent symlinks instead
+            };
+
+            let full_path = dest.join(rel_path);
+
+            // Ensure parent directory exists
+            if let Some(parent) = full_path.parent() {
+                std::fs::create_dir_all(parent)?;
+            }
+
+            // Write file
+            let file_size = file_data.data.len() as u64;
+            match std::fs::write(&full_path, &file_data.data) {
+                Ok(_) => {
+                    bytes_transferred += file_size;
+                    if local_map.contains_key(rel_path) {
+                        files_updated += 1;
+                    } else {
+                        files_created += 1;
+                    }
+                    // Finish transfer progress
+                    if let Some(ref progress) = progress {
+                        progress.finish_transfer(&PathBuf::from(rel_path), file_size);
+                    }
+                    session.send_file_done(*idx, 0).await?;
+                }
+                Err(e) => {
+                    tracing::warn!("Failed to write {}: {}", full_path.display(), e);
+                    session.send_file_done(*idx, 2).await?; // STATUS_WRITE_ERROR
+                }
             }
         }
     }
 
     // Step 4: Receive and create symlinks (if any)
-    // Note: The read_file_data returns None when it sees SYMLINK_BATCH
-    // We need to read the symlink batch body
-    if files_to_receive.is_empty() || files_created + files_updated < files_to_receive.len() as u64
+    // Note: In dry-run mode, we've told server to skip everything, so no symlinks sent
+    if !dry_run
+        && (files_to_receive.is_empty()
+            || files_created + files_updated < files_to_receive.len() as u64)
     {
         // Try to read symlink batch (may have been signaled by read_file_data returning None)
         match session.read_symlink_batch_body().await {
@@ -543,7 +856,6 @@ pub async fn sync_pull_server_mode(source: &SyncPath, dest: &Path) -> Result<Syn
                 tracing::debug!("Received {} symlinks", symlink_batch.entries.len());
                 let mut created = 0u32;
                 let mut failed: Vec<(String, String)> = Vec::new();
-
                 for entry in &symlink_batch.entries {
                     let link_path = dest.join(&entry.path);
 
@@ -589,6 +901,36 @@ pub async fn sync_pull_server_mode(source: &SyncPath, dest: &Path) -> Result<Syn
         duration
     );
 
+    // Mark progress as finished
+    if let Some(ref progress) = progress {
+        progress.mark_finished();
+    }
+
+    // Build detailed dry-run info if in dry-run mode
+    let dry_run_details = if dry_run {
+        // Estimate network bytes
+        let estimated_network_bytes: u64 = file_changes.iter().map(|fc| fc.transfer_bytes).sum();
+
+        // Estimate duration (conservative: 10 MB/s)
+        let avg_speed_bytes_per_sec = 10 * 1024 * 1024;
+        let estimated_duration_secs = if estimated_network_bytes > 0 {
+            (estimated_network_bytes as f64 / avg_speed_bytes_per_sec as f64).max(0.1)
+        } else {
+            0.0
+        };
+
+        Some(DryRunDetails {
+            file_changes,
+            directory_changes: dir_changes,
+            symlink_changes,
+            filtered_files: Vec::new(),
+            estimated_duration_secs,
+            estimated_network_bytes,
+        })
+    } else {
+        None
+    };
+
     Ok(SyncStats {
         files_scanned: file_list.entries.len() as u64,
         files_created,
@@ -600,6 +942,10 @@ pub async fn sync_pull_server_mode(source: &SyncPath, dest: &Path) -> Result<Syn
         errors: Vec::new(),
         dirs_created,
         symlinks_created,
+        bytes_would_add,
+        bytes_would_change,
+        bytes_would_delete: 0,
+        dry_run_details,
         ..Default::default()
     })
 }
@@ -680,6 +1026,7 @@ async fn scan_local_dest(dest: &Path) -> Result<Vec<SourceEntry>> {
 async fn process_delta_batch(
     session: &mut ServerSession,
     pending: &[(u32, &SourceEntry, u32)],
+    progress: Option<&Arc<ProgressState>>,
 ) -> Result<(u64, u64)> {
     use crate::server::protocol::ChecksumResp;
 
@@ -760,6 +1107,11 @@ async fn process_delta_batch(
         let (idx, ops, delta_bytes) = result?;
         let entry = pending[i].1;
 
+        // Start transfer progress
+        if let Some(progress) = progress {
+            progress.start_transfer(PathBuf::from(&entry.rel_path));
+        }
+
         bytes_transferred += delta_bytes;
         sent_indices.push((idx, entry.rel_path.clone(), delta_bytes, entry.size));
 
@@ -780,6 +1132,10 @@ async fn process_delta_batch(
             );
         } else {
             files_updated += 1;
+            // Finish transfer progress
+            if let Some(progress) = progress {
+                progress.finish_transfer(&PathBuf::from(&rel_path), size);
+            }
             tracing::debug!(
                 "Delta sync {}: sent {}KB (file is {}KB)",
                 rel_path,
